

import os
import re
import arrow

from kivy.uix.image import Image
from kivy.uix.label import Label
from kivy.uix.boxlayout import BoxLayout

from hoshi.messages import TranslatableStructuredMessage

from kivy_garden.ebs.core.labels import SelfScalingLabel
from kivy_garden.ebs.clocks import SimpleDigitalClock

from ebs.linuxnode.i18n.mixin import i18nMixin
from ebs.linuxnode.gui.kivy.timetable.base import Timetable
from ebs.linuxnode.gui.kivy.timetable.base import TimetableEntry

from ebs.linuxnode.gui.kivy.timetable.base import BaseTimetableMixin
from ebs.linuxnode.gui.kivy.timetable.base import BaseTimetableGuiMixin

from ebs.linuxnode.tables.translatable import TranslatableTableSpec
from ebs.linuxnode.tables.spec import BasicColumnSpec


def render_time(ts, fmt="HH:mm"):
    return ts.format(fmt)


route_rex = re.compile(r"(?P<origin>[\S -]+[\S])\s+to\s+(?P<destination>[\S -]+?[\S])($|\s+via\s+(?P<via>[\S]+))",
                       re.IGNORECASE)


class RsrtcRoute(TranslatableStructuredMessage):
    def __init__(self, route):
        m = route_rex.match(route.strip())
        if not m:
            print("Error Parsing : {}".format(route))
            parts = {'raw': route}
        else:
            parts = self.preprocess_parts(m.groupdict())
            parts['raw'] = route
        super(RsrtcRoute, self).__init__(parts)
        self.install_template('en_IN', "[b]{origin}[/b] to [b]{destination}[/b] via [b]{via}[/b]")
        self.install_template('en_IN', "[b]{origin}[/b] to [b]{destination}[/b]")
        self.install_template('en_IN', "{raw}")
        self.install_template('hi_IN', "[b]{origin}[/b] से [b]{destination}[/b] ([b]{via}[/b] के रास्ते)")
        self.install_template('hi_IN', "[b]{origin}[/b] से [b]{destination}[/b]")
        self.install_template('hi_IN', "{raw}")

    def preprocess_parts(self, parts):
        parts = self.preprocess_part(parts, 'origin')
        parts = self.preprocess_part(parts, 'destination')
        parts = self.preprocess_part(parts, 'via')
        return parts

    def preprocess_part(self, parts, key):
        if key in parts.keys():
            if parts[key]:
                parts[key] = parts[key].upper().strip()
            else:
                parts.pop(key)
        return parts


class RsrtcTimetableEntry(TimetableEntry):
    def __init__(self, data):
        super(RsrtcTimetableEntry, self).__init__(data)

    @property
    def service_id(self):
        return self.data['bus_service_no']

    @property
    def name(self):
        return RsrtcRoute(self.data['route_name'])

    @property
    def ts_start(self):
        return arrow.get(self.data['scheduled_date'], 'YYYY-MM-DD HH:mm:ss').replace(tzinfo="+05:30")

    @property
    def departure_time(self):
        return self.ts_start

    @property
    def status(self):
        if (arrow.now() - self.ts_start).total_seconds() > 900:
            return "DEPART"
        else:
            return ""

    @property
    def ts_end(self):
        return arrow.get(self.data['scheduled_date'], 'YYYY-MM-DD HH:mm:ss').replace(tzinfo="+05:30")
    
    @property
    def route_no(self):
        return self.data['route_no']

    @property
    def bus_type(self):
        return self.data['bus_type_cd']

    @property
    def bus_no(self):
        return self.data['bus_no'] or ''

    @property
    def depot(self):
        return self.data['depot_cd']

    def __repr__(self):
        return "<{0} {1} {2}>".format(self.__class__.__name__, self.departure_time, self.name)


class RsrtcTimetable(Timetable):
    def __init__(self, node):
        self._title = None
        spec = TranslatableTableSpec(
            self,
            [
             BasicColumnSpec("Service", 'bus_type', width=120,
                             font_bold=False, dynamic_scaling=False),
             BasicColumnSpec("Departs", "departure_time", width=140,
                             dynamic_scaling=False),
             BasicColumnSpec("Bus Number", 'bus_no', width=280,
                             font_bold=False, i18n=False, dynamic_scaling=False),
             BasicColumnSpec("Route", 'name', halign='left',
                             markup=True, font_bold=False),
             BasicColumnSpec("Depot", 'depot', width=120, font_bold=False,
                             i18n=False, dynamic_scaling=False)],
            show_column_header=True, dedup_keys=['service_id'],
            row_height=90, row_spacing=10, font_size='42sp', font_bold=True,
            name='RSRTC', languages=node.i18n_supported_languages,
            **node.text_font_params,
        )
        super(RsrtcTimetable, self).__init__(node, spec)
        metadata = {
            'project': "RSRTC Infopanel",
            'version': self.node.rsrtc_version,
            'msgid_bugs_address': "shashank.chintalagiri@gmail.com",
            'language_team': "StarXMedia Signage Maintenance Team",
            'last_translator': None,
            'copyright_holder': "Propress Instrumentation & Solutions Pvt. Ltd., India",
        }
        spec.install_metadata(metadata)
        spec.install_catalog_dir(os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir, 'locale')))
        self.install()

    @property
    def prior_window(self):
        return self.node.config.rsrtc_window_prior or 30

    @property
    def post_window(self):
        return self.node.config.rsrtc_window_post or 60

    def _build_title_text(self):
        title_text = BoxLayout(
            orientation='vertical',
            size_hint=(1, 1),
            spacing=0,
        )

        l_font_params = self.spec.font_params
        l_font_params.update({
            'font_size': '42sp',
            'bold': True
        })
        org_title = Label(
            text=self.preprocess("Rajasthan State Road Transport Corporation".upper()),
            color=self.palette.cell_background,
            size_hint=(1, None), height=55, valign='middle', halign='left',
            **l_font_params
        )
        org_title.bind(size=org_title.setter('text_size'))
        title_text.add_widget(org_title)

        l_font_params = self.spec.font_params
        l_font_params.update({
            'font_size': '32sp',
            'bold': True
        })
        station_name = SelfScalingLabel(
            text=self.preprocess(self._node.config.rsrtc_stop_name),
            color=self.palette.header_cell_background,
            size_hint=(1, None), height=45, valign='middle', halign='right',
            **l_font_params
        )
        station_name.bind(size=station_name.setter('text_size'))
        title_text.add_widget(station_name)
        return title_text

    def _redraw_title(self, *_):
        _new_title_text = self._build_title_text()
        self._title.remove_widget(self._title.children[1])
        self._title.add_widget(_new_title_text, 1)

    def _build_title(self):
        if self._title:
            return self._title

        self._title = BoxLayout(
            orientation='horizontal', spacing=20, padding=[30, 10],
            size_hint=(1, None), height=120,
        )

        logo = Image(
            source=self.node.config.get_path(os.path.join('resources', 'logo.png')),
            size_hint=(None, 1), allow_stretch=True, keep_ratio=True)
        self._title.add_widget(logo)

        self._title_text = self._build_title_text()
        self._title.add_widget(self._title_text)

        # station_code = SelfScalingLabel(
        #     text=self._node.config.rsrtc_stop_id.upper(),
        #     color=self.palette.header_cell_background,
        #     size_hint=(None, 1), font_size='110sp', bold=True,
        #     valign='middle', halign='right',  # font_name=self.spec.font_name
        # )
        # station_code.bind(texture_size=station_code.setter('size'))
        # self._title.add_widget(station_code)

        l_font_params = self.spec.font_params
        l_font_params.update({
            'font_size': '110sp',
            'bold': True
        })
        clock = SimpleDigitalClock(
            color=self.palette.cell_background,
            size_hint=(None, 1), valign='middle', halign='right',
            **l_font_params
        )
        clock.bind(texture_size=clock.setter('size'))
        clock.start()
        self._title.add_widget(clock)
        return self._title


class RsrtcTimetableMixin(BaseTimetableGuiMixin, BaseTimetableMixin, i18nMixin):
    _timetable_class = RsrtcTimetable
    _timetable_entry_class = RsrtcTimetableEntry

    def __init__(self, *args, **kwargs):
        super(RsrtcTimetableMixin, self).__init__(*args, **kwargs)

    def install(self):
        super(RsrtcTimetableMixin, self).install()
        self._timetable.fallback_image = self.config.get_path('resources/rsrtc-bg-blue.png')
        self.i18n.install_object_translator(arrow.Arrow, render_time)

    def rsrtc_timetable_update(self, response):
        self.log.info("Loading Timetable Response with {n} Entries",
                      n=len(response))
        self.timetable_update(response, incremental=False)

    @property
    def rsrtc_timetable(self):
        return self.timetable_gui





from twisted.internet.task import LoopingCall
from starxmedia.api.core import SXMApiEngineCore


class SXMApiExtentionRSRTC(SXMApiEngineCore):
    _api_ep_device_rsrtc_settings = 'iot/load-device-rsrtc-details'

    def __init__(self, *args, **kwargs):
        super(SXMApiExtentionRSRTC, self).__init__(*args, **kwargs)
        self._api_rsrtc_settings_task = None

    def _api_apply_device_rsrtc_settings(self, settings):
        self.log.info("Got device rsrtc settings from the server : {settings}",
                      settings=settings)
        self.config.rsrtc_stop_id = settings['stop_code']
        self.config.rsrtc_stop_name = settings['stop_name']
        if 'window_prior' in settings.keys():
            self.config.rsrtc_window_prior = settings['window_prior']
        if 'window_post' in settings.keys():
            self.config.rsrtc_window_post = settings['window_post']

    def _api_device_rsrtc_settings_handler(self, response):
        self._api_apply_device_rsrtc_settings(response['data'])

    def api_get_device_rsrtc_settings(self):
        return self._api_execute(
            self._api_ep_device_rsrtc_settings,
            self._api_basic_params,
            self._api_device_rsrtc_settings_handler
        )

    @property
    def api_rsrtc_settings_task(self):
        if self._api_rsrtc_settings_task is None:
            self._api_rsrtc_settings_task = LoopingCall(self.api_get_device_rsrtc_settings)
        return self._api_rsrtc_settings_task





from .timetable import RsrtcTimetableEngine


class RsrtcApiEngine(RsrtcTimetableEngine):
    pass


from starxmedia.api import SXMApiEngine
from .sxmext import SXMApiExtentionRSRTC


class SXMApiEngineRSRTC(SXMApiExtentionRSRTC, SXMApiEngine):
    pass



""" configuration """
import os
import re
import warnings

from .utils import trace

DEFAULT_TAG_REGEX = r"^(?:[\w-]+-)?(?P<version>[vV]?\d+(?:\.\d+){0,2}[^\+]*)(?:\+.*)?$"
DEFAULT_VERSION_SCHEME = "guess-next-dev"
DEFAULT_LOCAL_SCHEME = "node-and-date"


def _check_tag_regex(value):
    if not value:
        value = DEFAULT_TAG_REGEX
    regex = re.compile(value)

    group_names = regex.groupindex.keys()
    if regex.groups == 0 or (regex.groups > 1 and "version" not in group_names):
        warnings.warn(
            "Expected tag_regex to contain a single match group or a group named"
            " 'version' to identify the version part of any tag."
        )

    return regex


def _check_absolute_root(root, relative_to):
    trace("l", repr(locals()))
    if relative_to:
        if os.path.isabs(root) and not root.startswith(relative_to):
            warnings.warn(
                "absolute root path '%s' overrides relative_to '%s'"
                % (root, relative_to)
            )
        if os.path.isdir(relative_to):
            warnings.warn(
                "relative_to is expected to be a file,"
                " its the directory %r\n"
                "assuming the parent directory was passed" % (relative_to,)
            )
            trace("dir", relative_to)
            root = os.path.join(relative_to, root)
        else:
            trace("file", relative_to)
            root = os.path.join(os.path.dirname(relative_to), root)
    return os.path.abspath(root)


class Configuration:
    """ Global configuration model """

    def __init__(
        self,
        relative_to=None,
        root=".",
        version_scheme=DEFAULT_VERSION_SCHEME,
        local_scheme=DEFAULT_LOCAL_SCHEME,
        write_to=None,
        write_to_template=None,
        tag_regex=DEFAULT_TAG_REGEX,
        parentdir_prefix_version=None,
        fallback_version=None,
        fallback_root=".",
        parse=None,
        git_describe_command=None,
        dist_name=None,
    ):
        # TODO:
        self._relative_to = relative_to
        self._root = "."

        self.root = root
        self.version_scheme = version_scheme
        self.local_scheme = local_scheme
        self.write_to = write_to
        self.write_to_template = write_to_template
        self.parentdir_prefix_version = parentdir_prefix_version
        self.fallback_version = fallback_version
        self.fallback_root = fallback_root
        self.parse = parse
        self.tag_regex = tag_regex
        self.git_describe_command = git_describe_command
        self.dist_name = dist_name

    @property
    def fallback_root(self):
        return self._fallback_root

    @fallback_root.setter
    def fallback_root(self, value):
        self._fallback_root = os.path.abspath(value)

    @property
    def absolute_root(self):
        return self._absolute_root

    @property
    def relative_to(self):
        return self._relative_to

    @relative_to.setter
    def relative_to(self, value):
        self._absolute_root = _check_absolute_root(self._root, value)
        self._relative_to = value
        trace("root", repr(self._absolute_root))
        trace("relative_to", repr(value))

    @property
    def root(self):
        return self._root

    @root.setter
    def root(self, value):
        self._absolute_root = _check_absolute_root(value, self._relative_to)
        self._root = value
        trace("root", repr(self._absolute_root))
        trace("relative_to", repr(self._relative_to))

    @property
    def tag_regex(self):
        return self._tag_regex

    @tag_regex.setter
    def tag_regex(self, value):
        self._tag_regex = _check_tag_regex(value)

    @classmethod
    def from_file(cls, name="pyproject.toml", dist_name=None):
        """
        Read Configuration from pyproject.toml (or similar).
        Raises exceptions when file is not found or toml is
        not installed or the file has invalid format or does
        not contain the [tool.setuptools_scm] section.
        """
        with open(name) as strm:
            defn = __import__("toml").load(strm)
        section = defn.get("tool", {})["setuptools_scm"]
        return cls(dist_name=dist_name, **section)



#!python

import faulthandler

from ebs.linuxnode.gui.kivy.utils.launcher import prepare_config
from ebs.linuxnode.gui.kivy.utils.launcher import prepare_environment
from ebs.linuxnode.gui.kivy.utils.launcher import prepare_kivy


def run_node():
    nodeconfig = prepare_config('rsrtc')

    prepare_environment(nodeconfig)
    prepare_kivy(nodeconfig)

    from ebs.linuxnode.core import config
    config.current_config = nodeconfig

    from .app import RsrtcApplication

    print("Creating Application : {}".format(RsrtcApplication))
    app = RsrtcApplication(config=nodeconfig)
    app.run()


if __name__ == '__main__':
    print("Starting faulthandler")
    faulthandler.enable()
    run_node()





from twisted.internet.task import LoopingCall
from twisted.internet import reactor
from .core import RsrtcApiCoreEngine


class RsrtcTimetableEngine(RsrtcApiCoreEngine):
    _api_ep_timetable = 'bus-schedules'

    def __init__(self, *args, **kwargs):
        super(RsrtcTimetableEngine, self).__init__(*args, **kwargs)
        self._retries = 0
        self._rapid_checks = 0
        self._api_timetable_task = None

    def _api_timetable_handler(self, response):
        self.log.info("Got Timetable API Response with {n} Entries",
                      n=len(response))
        if len(response):
            self._retries = 0
            self._rapid_checks = 0
            self._actual.rsrtc_timetable_update(response)
        else:
            self._retries += 1
            if self._retries >= 3:
                self._rapid_checks += 1
                if self._rapid_checks < 10:
                    reactor.callLater(60, self.api_get_timetable)
                self._actual.rsrtc_timetable_update(response)
            else:
                self.api_get_timetable()

    def _api_timetable_params(self, token):
        rv = self._api_basic_params(token)
        rv['_query'].append(('code', self.config.rsrtc_stop_id))
        return rv

    def api_get_timetable(self):
        return self._api_execute(
            self._api_ep_timetable,
            self._api_timetable_params,
            self._api_timetable_handler
        )

    """ Timetable API Management Task """
    @property
    def api_timetable_task(self):
        if self._api_timetable_task is None:
            self._api_timetable_task = LoopingCall(self.api_get_timetable)
        return self._api_timetable_task




__author__ = 'Chintalagiri Shashank <shashank.chintalagiri@gmail.com>'

from pkg_resources import get_distribution, DistributionNotFound
try:
    __version__ = get_distribution('infopanel-rsrtc').version
except DistributionNotFound:
    # package is not installed
    from setuptools_scm import get_version
    __version__ = get_version(root='../', relative_to=__file__)





import os
import pkg_resources

from kivy_garden.ebs.core.colors import GuiPalette
from ebs.linuxnode.core.config import ElementSpec, ItemSpec

from starxmedia.node import StarXMediaNode

from .timetable.rsrtc import RsrtcTimetableMixin

from .api import RsrtcApiEngine
from .api import SXMApiEngineRSRTC


class RsrtcNode(RsrtcTimetableMixin, StarXMediaNode):
    _palette = GuiPalette(
        background=(0xf7 / 255, 0xf7 / 255, 0xf7 / 255),
        foreground=(0xff / 255, 0xff / 255, 0xff / 255),
        color_1=(0x72 / 255., 0xc2 / 255., 0xf5 / 255., 1),
        color_2=(0x07 / 255., 0x3e / 255., 0xbc / 255., 1)
    )
    _gui_marquee_bgcolor = (0xff / 255., 0x00 / 255., 0x00 / 255., 0.6)
    _gui_supports_overlay_mode = True

    _appname = "rsrtc"
    _supported_languages = ['en_IN', 'hi_IN']
    _default_rsrtc_api_url = 'http://rsrtc.starxmedia.in:9091'

    def __init__(self, *args, **kwargs):
        super(RsrtcNode, self).__init__(*args, **kwargs)

    @property
    def rsrtc_version(self):
        return pkg_resources.get_distribution('infopanel-rsrtc').version

    def install(self):
        self.config.register_application_root(
            os.path.abspath(os.path.dirname(__file__))
        )

        sxm_api_engine = SXMApiEngineRSRTC(self)
        self.modapi_install(sxm_api_engine)

        super(RsrtcNode, self).install()

        _elements = {
            'rsrtc_api_url': ElementSpec('rsrtc', 'url', ItemSpec(str, fallback=self._default_rsrtc_api_url)),
            'rsrtc_stop_id': ElementSpec('rsrtc', 'stop_id', ItemSpec(str, fallback='JPR', read_only=False)),
            'rsrtc_stop_name': ElementSpec('rsrtc', 'stop_name', ItemSpec(str, fallback="", read_only=False)),
            'rsrtc_window_prior': ElementSpec('rsrtc', 'window_prior', ItemSpec(int, fallback=30, read_only=False)),
            'rsrtc_window_post': ElementSpec('rsrtc', 'window_post', ItemSpec(int, fallback=60, read_only=False)),
        }

        for name, spec in _elements.items():
            self.config.register_element(name, spec)

        rsrtc_api_engine = self.modapi_engine('rsrtc')
        if not rsrtc_api_engine:
            self.log.info("Installing Vanilla RSRTC Api Engine")
            rsrtc_api_engine = RsrtcApiEngine(self)
            self.modapi_install(rsrtc_api_engine)

        sxm_api_engine.install_task('api_rsrtc_settings_task', 3600)
        rsrtc_api_engine.install_task('api_timetable_task', 300)

    def start(self):
        super(RsrtcNode, self).start()

    def stop(self):
        super(RsrtcNode, self).stop()





import faulthandler
from memory_profiler import profile
from rsrtc.runnode import run_node


@profile
def run_profile():
    run_node()


if __name__ == '__main__':
    print("Starting faulthandler")
    faulthandler.enable()

    print("Running for memory profiling")
    run_profile()



import setuptools

_requires = [
    'signagenode>=2.7.2',
    'setuptools-scm',
    'ebs-linuxnode-tables',
    'ebs-linuxnode-i18n',
]

setuptools.setup(
    name='signagenode-rsrtc',
    url='',
    use_scm_version=True,

    author='Chintalagiri Shashank',
    author_email='shashank.chintalagiri@gmail.com',

    description='',
    long_description='',

    packages=setuptools.find_packages(),
    package_data={'rsrtc': ['default/config.ini',
                            'default/background.png',
                            'resources/logo.png',
                            'resources/rsrtc-bg-blue.png',
                            'resources/rsrtc-bg-white.png',
                            'locale/*',
                            'locale/*/LC_MESSAGES/*']},

    install_requires=_requires,

    setup_requires=['setuptools_scm'],

    entry_points={
          'console_scripts': [
              'rsrtc = rsrtc.runnode:run_node'
          ]
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX :: Linux',
    ],
    include_package_data=True,
    zip_safe=False,
)





import arrow
from twisted.internet import reactor
from twisted.internet.task import LoopingCall

from rsrtc import api
from ebs.iot.linuxnode.modapi.standalone import StandaloneActual
from ebs.iot.linuxnode.modapi.standalone import StandaloneConfig


stop_id = 'JPR'


class StandaloneActualRsrtc(StandaloneActual):
    def rsrtc_timetable_update(self, response):
        nitems = len(response)
        items = [x['routeNo'] for x in response]
        ditems = set(items)
        ndistinct = len(ditems)
        print("{} : {:>3} results : {:>3} distinct : {}".format(
            arrow.now(), nitems, ndistinct, ditems))


class StandaloneConfigRsrtc(StandaloneConfig):
    def __init__(self, *args, **kwargs):
        self.rsrtc_stop_id = kwargs.pop('stop_id', 'KTH')
        super(StandaloneConfigRsrtc, self).__init__(*args, **kwargs)
        self.rsrtc_api_url = 'http://180.92.168.54:8082/RsrtcThirdPartyService'


config = StandaloneConfigRsrtc(stop_id=stop_id)
e = api.RsrtcApiEngine(StandaloneActualRsrtc(), config=config)


def get_results_summary():
    e.api_get_timetable()


if __name__ == '__main__':
    period = 5.0
    print("API Response Summary Test:")
    print(" API Url : {}".format(config.rsrtc_api_url))
    print(" Stop Code: {}".format(config.rsrtc_stop_id))
    print(" Period: {}s".format(period))
    loop = LoopingCall(get_results_summary)
    loopDeferred = loop.start(period)
    reactor.run()




from twisted.internet.defer import succeed

from ebs.linuxnode.modapi.engine import ModularHttpApiEngine


class RsrtcApiCoreEngine(ModularHttpApiEngine):
    _prefix = "rsrtc"
    _api_probe = 'api_test_connection'
    _api_tasks = []
    _api_reconnect_frequency = 30
    _api_baseurl = 'config:rsrtc_api_url'
    _api_headers = {'Content-Type': 'application/json'}

    _api_ep_test_connection = 'bus-schedules'

    @property
    def api_token(self):
        return succeed(None)

    def api_token_reset(self):
        return succeed(None)

    def _api_basic_params(self, token):
        return {'_query': []}

    def _api_test_query_params(self, token):
        rv = self._api_basic_params(token)
        rv['_query'].append(('code', 'KTH'))
        return rv

    def _api_ep_test_connection_handler(self, response):
        return succeed(True)

    def api_test_connection(self):
        return self._api_execute(
            self._api_ep_test_connection,
            self._api_test_query_params,
            self._api_ep_test_connection_handler
        )
